// This header defines advanced debug methods for Darkwire Software.
#include <stdio.h>
#include <windows.h>
#include <string>

#ifdef USE_ADVANCED_DEBUG
#ifdef DEBUG_LEVEL

// Highest amount of verbosity
#if (DEBUG_LEVEL >= 2)
#define UltraDebug(a, ...) TrueFunc(__LINE__, __FILE__, 3, a, __VA_ARGS__)
#else
#define UltraDebug(...) 
#endif

// Average verbosity
#if (DEBUG_LEVEL >= 1)
#define IncDebug(a, ...) TrueFunc(__LINE__, __FILE__, 2, a, __VA_ARGS__)
#else
#define IncDebug(...) 
#endif

// Least verbosity - do not use except in debug situations
#if (DEBUG_LEVEL >= 0)
#define Debug(a, ...) TrueFunc(__LINE__, __FILE__, 1, a, __VA_ARGS__)
#else
#pragma warning (disable:4390)
#define Debug(...) 
#pragma warning (default:4390)
#endif

// #define is used for debug; infinite while loop enabled
#if defined(USE_DEFINE_AS_DEBUG) && defined(HALT_ON_DEBUG_MESSAGE) // USE_DEFINE_AS_DEBUG is enabled
#pragma message("Warning: USE_DEFINE_AS_DEBUG is a multi-line macro. This can cause errors if it is used as a single-line macro.")
#define TrueFunc(line, file, level, output, ...) \
{ \
	FILE * f = fopen("C:\\ExtractedServerLog.txt", "ab"); \
	unsigned char c = 0; \
	while(!f) \
	{ \
		f = fopen("C:\\ExtractedServerLog.txt", "ab"); \
		Sleep(20);
		if(c++ == MAXUCHAR)
		{
			MessageBoxA(NULL, "Could open file handle to log!", "Extracted server - Debug info", MB_OK|MB_ICONERROR); \
			break; \
		} \
	} \
	if(f) \
	{ \
		string s(file), t = s.substr(s.rfind("\\"), s.length()); \
		fprintf(f, "%u:\t%u.4\t%s\t", level, line, t.c_str()); \
		fprintf(f, output, __VA_ARGS__); \
		fputs("\r\n", f); \
		fclose(f); \
	} \
	while(true) \
		Sleep(500); \
}
#endif // USE_DEFINE_AS_DEBUG defined; HALT_ON_DEBUG_MESSAGE defined

// #define is used for debug; infinite while loop disabled
#if defined(USE_DEFINE_AS_DEBUG) && !defined(HALT_ON_DEBUG_MESSAGE) // HALT_ON_DEBUG_MESSAGE undefined
#pragma message("Warning: USE_DEFINE_AS_DEBUG is a multi-line macro. This can cause errors if it is used as a single-line macro.")
#define TrueFunc(line, file, level, output, ...) \
{ \
	FILE * f = fopen("C:\\ExtractedServerLog.txt", "ab"); \
	unsigned char c = 0; \
	while(!f) \
	{ \
		f = fopen("C:\\ExtractedServerLog.txt", "ab"); \
		Sleep(20);
		if(c++ == MAXUCHAR)
		{
			MessageBoxA(NULL, "Could open file handle to log!", "Extracted server - Debug info", MB_OK|MB_ICONERROR); \
			break; \
		} \
	} \
	if(f) \
	{ \
		string s(file), t = s.substr(s.rfind("\\"), s.length()); \
		fprintf(f, "%u:\t%u.4\t%s\t", level, line, t.c_str()); \
		fprintf(f, output, __VA_ARGS__); \
		fputs("\r\n", f); \
		fclose(f); \
	} \
}
#endif // USE_DEFINE_AS_DEBUG defined; HALT_ON_DEBUG_MESSAGE undefined

#else // DEBUG_LEVEL undefined
#error USE_ADVANCED_DEBUG invoked, but no debug level setting. Use #define DEBUG_LEVEL 0-3 (less to more output);
#endif

#else // USE_ADVANCED_DEBUG undefined
#define TrueFunc()
#define Debug()
#define IncDebug()
#define UltraDebug()
#endif