#include "common.h"
// ==============================================================================================================================================================
// FILE SERVER >>> CONSTRUCTORS/DECONSTRUCTORS
// ==============================================================================================================================================================

// Constructor
SERVER_FILE::SERVER_FILE(Extension * E, char * FlashPolicyPath)
{
	// Copy server type to correct variable
	strcpy_s(ServerType, 5, "ESB1");

	// Allow server functions to call GenerateEvent
	pExt = E;
	Debug("Attempting to boot Lacewing server.");
	// Acquire lock
	Lacewing::SpinSync::WriteLock ThreadLock(Lock);
	
	LacewingServer = new Lacewing::RelayServer(pExt->AppEventPump);
	LacewingServer->Tag = this;
	LacewingServer->onChannelMessage(::OnChannelMessage);
	LacewingServer->onConnect(::OnConnect);
	LacewingServer->onDisconnect(::OnDisconnect);
	LacewingServer->onError(::OnError);
	LacewingServer->onJoinChannel(::OnJoinChannel);
	LacewingServer->onLeaveChannel(::OnLeaveChannel);
	LacewingServer->onPeerMessage(::OnPeerMessage);
	LacewingServer->onServerMessage(::OnServerMessage);
	LacewingServer->onSetName(::OnSetName);

	LacewingServer->SetWelcomeMessage("Generic welcome message");
	LacewingServer->Host(6121);
	
	FlashPortal = new Lacewing::FlashPolicy(pExt->AppEventPump);
	FlashPortal->Tag = this;
	FlashPortal->onError(::OnError);
	FlashPortal->Host(FlashPolicyPath);

	SendingThread = CreateThread(NULL, NULL, (LPTHREAD_START_ROUTINE)FileServer_TimedTransferThread, this, NULL, NULL);

	Debug("FILE_SERVER::FILE_SERVER() executed; host returns %i for Hosting, %i for Flash Hosting, %p for SendingThread location.", LacewingServer->Hosting(), FlashPortal->Hosting(), SendingThread);
	
	free(FlashPolicyPath);
	ThreadLock.Release();
}

// Deconstructor
SERVER_FILE::~SERVER_FILE()
{
	// A side note, should I call LacewingServer->Unhost() here?
}

// ==============================================================================================================================================================
// FILE SERVER >>> LACEWING LIBRARY FUNCTIONS
// ==============================================================================================================================================================
bool SERVER_FILE::OnConnect(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client)
{
	// Content stripped
	Debug("SERVER_FILE::OnConnect()");
	pExt->Runtime.GenerateEvent(MF2C_CONNECTION_FOR_SERVER);
	return true;
}

void SERVER_FILE::OnDisconnect(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client)
{
	// Content stripped
	Debug("SERVER_FILE::OnDisconnect");
	pExt->Runtime.GenerateEvent(MF2C_DISCONNECTION_FOR_SERVER);
}

void SERVER_FILE::OnError(Lacewing::RelayServer &Server, Lacewing::Error &Error)
{
	// Content stripped
	Debug("Error occured with RelayServer (regular): [%s].", Error.ToString());
	pExt->Runtime.GenerateEvent(MF2C_ERROR_LOCAL);
}	

void SERVER_FILE::OnError(Lacewing::FlashPolicy &FlashPolicy, Lacewing::Error &Error)
{
	// Content stripped
	Debug("Error occured with RelayServer (flash portal): [%s].", Error.ToString());
	pExt->Runtime.GenerateEvent(MF2C_ERROR_LOCAL);
}

void SERVER_FILE::OnServerMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, bool Blasted, int Subchannel, char * Data, int Size, int Variant)
{
	// Content stripped
	Debug("SERVER_FILE::OnServerMessage");
}

bool SERVER_FILE::OnChannelMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel, bool Blasted, int Subchannel, char * Data, int Size, int Variant)
{
	// Content stripped
	Debug("SERVER_FILE::OnChannelMessage");
	return true;
}

bool SERVER_FILE::OnPeerMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel, Lacewing::RelayServer::Client &TargetClient, bool Blasted, int Subchannel, char * Packet, int Size, int Variant)
{
	// Content stripped
	Debug("SERVER_FILE::OnPeerMessage");
	return true;
}

bool SERVER_FILE::OnJoinChannel(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel)
{
	// Content stripped
	Debug("SERVER_FILE::OnJoinChannel");
	return true;
}

bool SERVER_FILE::OnLeaveChannel(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel)
{
	// Content stripped
	Debug("SERVER_FILE::OnLeaveChannel");
	return true;
}

bool SERVER_FILE::OnSetName(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, const char * NewName)
{
	// Content stripped
	Debug("SERVER_FILE::OnSetName");
	return true;
}
// ==============================================================================================================================================================
// FILE SERVER >>> MMF2 FUNCTIONS
// ==============================================================================================================================================================

// Boot new server from MMF2
void Extension::FileServers_BootNewServer(char * FlashPolicy)
{
	// Default to what the called constructor specifies.
	char * CopyOfFlashPolicy = _strdup(FlashPolicy);
	UltraDebug("Passkey is %p [%s].", CopyOfFlashPolicy, CopyOfFlashPolicy);

	// Content stripped
	// All settings specified; boot the server.
	SERVER_FILE * FileServer = new SERVER_FILE(this, FlashPolicy); // This function runs free() on FlashPolicy
	// Make server variables accessible to MMF2
	DarknetLocalNodeIDs.push_back((void *)FileServer); 

	UltraDebug("Initiation of new file server is complete!");
}
// ==============================================================================================================================================================
// FILE SERVER >>> THREADS
// ==============================================================================================================================================================

// Created in FILE_SERVER::FILE_SERVER()
void WINAPI FileServer_TimedTransferThread(LPVOID a)
{
	// Content stripped
	while(true)
		Sleep(1000);
}
