// LacewingRelocater.cpp
// This file moves all generic Lacewing requests to the associated specific server/client function.
#include "common.h"

/* Format of SERVER's and CLIENT's first variable, char [5], contains:
	[0] = Always 'E'
	[1] = 'S' for server, 'C' for client
	[2] = 'A' for proxy server, 'B' for file server, etc.
	[3] = Always '1'
	[4] = Always '\0'
	Hence this Lacewing relocater can call the specific server functions without issues about not being
	able to use & on a member function pointer or whatever it moans about.
*/

bool ::OnSetName(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, const char * Name)
{
	Debug("Attempting to relocate OnSetName (Server).");
	char Type = *(char *)Server.Tag+1;
	char Node = *(char *)Server.Tag+2;
	// Is node a server?
	if(Type == 'S')
	{
		if(Node == 'A')	return ((SERVER_PROXY *)Server.Tag)->OnSetName(Server, Client, Name);
		if(Node == 'B')	return ((SERVER_FILE *)Server.Tag)->OnSetName(Server, Client, Name);
	}
	Debug("Error with RelayServer's Set Name - node is not a server.");
	return false;
}
bool ::OnConnect(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client)
{
	Debug("Attempting to relocate OnConnect (Server).");
	char Type = *(char *)Server.Tag+1;
	char Node = *(char *)Server.Tag+2;
	// Is node a server?
	if(Type == 'S')
	{
		if(Node == 'A')	return ((SERVER_PROXY *)Server.Tag)->OnConnect(Server, Client);
		if(Node == 'B')	return ((SERVER_FILE *)Server.Tag)->OnConnect(Server, Client);
	}
	Debug("Error with RelayServer's On Connect - node is not a server.");
	return false;
}
void ::OnDisconnect(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client)
{
	Debug("Attempting to relocate OnDisconnect (RelayServer).");
	char Type = *(char *)Server.Tag+1;
	char Node = *(char *)Server.Tag+2;
	// Is node a server?
	if(Type == 'S')
	{
		if(Node == 'A')	return ((SERVER_PROXY *)Server.Tag)->OnDisconnect(Server, Client);
		if(Node == 'B')	return ((SERVER_FILE *)Server.Tag)->OnDisconnect(Server, Client);
	}
	Debug("Error with RelayServer's On Disconnect - node is not a server.");
}
void ::OnError(Lacewing::RelayServer &Server, Lacewing::Error &Error)
{
	Debug("Attempting to relocate OnError (RelayServer, Regular).");
	char Type = *(char *)Server.Tag+1;
	char Node = *(char *)Server.Tag+2;
	// Is node a server?
	if(Type == 'S')
	{
		if(Node == 'A')	return ((SERVER_PROXY *)Server.Tag)->OnError(Server, Error);
		if(Node == 'B')	return ((SERVER_FILE *)Server.Tag)->OnError(Server, Error);
	}
	Debug("Error with RelayServer's On Error - node is not a server.");
}
void ::OnError(Lacewing::FlashPolicy &FlashPolicy, Lacewing::Error &Error)
{
	Debug("Attempting to relocate OnError (RelayServer, Flash).");
	char Type = *(char *)FlashPolicy.Tag+1;
	char Node = *(char *)FlashPolicy.Tag+2;
	// Is node a server?
	if(Type == 'S')
	{
		if(Node == 'A')	return ((SERVER_PROXY *)FlashPolicy.Tag)->OnError(FlashPolicy, Error);
		if(Node == 'B')	return ((SERVER_FILE *)FlashPolicy.Tag)->OnError(FlashPolicy, Error);
	}
	Debug("Error with RelayServer's On Error (FlashPolicy) - node is not a server [%s].", (char *)FlashPolicy.Tag);
}
void ::OnServerMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, bool Blasted, int Subchannel, char * Data, int Size, int Variant)
{
	Debug("Attempting to relocate OnServerMessage (RelayServer).");
	char Type = *(char *)Server.Tag+1;
	char Node = *(char *)Server.Tag+2;
	// Is node a server?
	if(Type == 'S')
	{
		if(Node == 'A')	return ((SERVER_PROXY *)Server.Tag)->OnServerMessage(Server, Client, Blasted, Subchannel, Data, Size, Variant);
		if(Node == 'B')	return ((SERVER_FILE *)Server.Tag)->OnServerMessage(Server, Client, Blasted, Subchannel, Data, Size, Variant);
	}
	Debug("Error with RelayServer's On Server Message - node is not a server.");
}
bool ::OnChannelMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel, bool Blasted, int Subchannel, char * Data, int Size, int Variant)
{
	Debug("Attempting to relocate OnChannelMessage (RelayServer).");
	char Type = *(char *)Server.Tag+1;
	char Node = *(char *)Server.Tag+2;
	// Is node a server?
	if(Type == 'S')
	{
		if(Node == 'A')	return ((SERVER_PROXY *)Server.Tag)->OnChannelMessage(Server, Client, Channel, Blasted, Subchannel, Data, Size, Variant);
		if(Node == 'B')	return ((SERVER_FILE *)Server.Tag)->OnChannelMessage(Server, Client, Channel, Blasted, Subchannel, Data, Size, Variant);
	}
	Debug("Error with RelayServer's On Channel Message - node is not a server.");
	return false;
}
bool ::OnPeerMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel, Lacewing::RelayServer::Client &TargetClient, bool Blasted, int Subchannel, char * Packet, int Size, int Variant)
{
	Debug("Attempting to relocate OnPeerMessage (RelayServer).");
	char Type = *(char *)Server.Tag+1;
	char Node = *(char *)Server.Tag+2;
	// Is node a server?
	if(Type == 'S')
	{
		if(Node == 'A')	return ((SERVER_PROXY *)Server.Tag)->OnPeerMessage(Server, Client, Channel, TargetClient, Blasted, Subchannel, Packet, Size, Variant);
		if(Node == 'B')	return ((SERVER_FILE *)Server.Tag)->OnPeerMessage(Server, Client, Channel, TargetClient, Blasted, Subchannel, Packet, Size, Variant);
	}
	Debug("Error with RelayServer's On Peer Message - node is not a server.");
	return false;
}
bool ::OnJoinChannel(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel)
{
	Debug("Attempting to relocate OnJoinChannel (RelayServer).");
	char Type = *(char *)Server.Tag+1;
	char Node = *(char *)Server.Tag+2;
	// Is node a server?
	if(Type == 'S')
	{
		if(Node == 'A')	return ((SERVER_PROXY *)Server.Tag)->OnJoinChannel(Server, Client, Channel);
		if(Node == 'B')	return ((SERVER_FILE *)Server.Tag)->OnJoinChannel(Server, Client, Channel);
	}
	Debug("Error with RelayServer's On Join Channel - node is not a server.");
	return false;
}
bool ::OnLeaveChannel(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel)
{
	Debug("Attempting to relocate OnLeaveChannel (RelayServer).");
	char Type = *(char *)Server.Tag+1;
	char Node = *(char *)Server.Tag+2;
	// Is node a server?
	if(Type == 'S')
	{
		if(Node == 'A')	return ((SERVER_PROXY *)Server.Tag)->OnLeaveChannel(Server, Client, Channel);
		if(Node == 'B')	return ((SERVER_FILE *)Server.Tag)->OnLeaveChannel(Server, Client, Channel);
	}
	Debug("Error with RelayServer's On Leave Channel - node is not a server.");
	return false;
}