#if !defined(TGFEXT) && !defined(PROEXT) && !defined(MMFEXT)
#include "common.h"
#endif
#include "Lacewing.h"

#include "ImageFlt.h"
#include "ImgFlt.h"
#include "CfcFile.h" 

// Include guard
#ifndef EXTRACTED_SERVER_H
#define EXTRACTED_SERVER_H

class Extension;

// List of condition IDs
const enum ConditionMenu {
	MF2C_CONNECTION_FOR_SERVER,
	MF2C_IDENTITY_SET_ALLOWED_FOR_SERVER,
	MF2C_IDENTITY_SET_DENIED_FOR_SERVER,
	MF2C_DISCONNECTION_FOR_SERVER,
	MF2C_ERROR_LOCAL,
};

// Server: Base server class - contains all server settings that are in every server type.
class SERVER {
public:
	// ==============================================================================================================================================================
	// ALL SERVERS >>> VARIABLES
	// ==============================================================================================================================================================
								// Content stripped
						   char ServerType [5];		// Server type: DSXX + \0. MUST BE FIRST VARIABLE.
					Extension * pExt;				// Pointer to Extension
			 Lacewing::SpinSync Lock;				// Thead-safe operations read & write lock
		Lacewing::RelayServer * LacewingServer;		// Lacewing server
		Lacewing::FlashPolicy * FlashPortal;		// Flash policy
	
	// The following definition, if enabled, means any (invalid) calls to SERVER's Lacewing functions will
	// report an error instead of crashing with a "pure virtual call" error.
	#ifdef PURE_VIRTUAL_DEFENCE
	#define Defence
	#else // No defence
	#define Defence =0
	#endif

	// Lacewing handlers - ABSTRACT - DO NOT CALL
	virtual bool OnConnect(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client) Defence;
	virtual void OnDisconnect(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client) Defence;
	virtual void OnError(Lacewing::RelayServer &Server, Lacewing::Error &Error) Defence;
	virtual void OnError(Lacewing::FlashPolicy &FlashPolicy, Lacewing::Error &Error) Defence;
	virtual void OnServerMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, bool Blasted, int Subchannel, char * Data, int Size, int Variant) Defence;
	virtual bool OnChannelMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel, bool Blasted, int Subchannel, char * Data, int Size, int Variant) Defence;
	virtual bool OnPeerMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel, Lacewing::RelayServer::Client &TargetClient, bool Blasted, int Subchannel, char * Packet, int Size, int Variant) Defence;
	virtual bool OnJoinChannel(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel) Defence;
	virtual bool OnLeaveChannel(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel) Defence;
	virtual bool OnSetName(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, const char * Name) Defence;
	
	#undef Defence
};

// Proxy server: Forwards communications
class SERVER_PROXY : public SERVER
{
public:
	//	Variables (stripped)
	int i;

	// Constructors
	SERVER_PROXY(unsigned char max, Extension * E);
	
	// Deconstructors
	~SERVER_PROXY();

	// Pass along
	bool OnConnect(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client);
	void OnDisconnect(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client);
	void OnError(Lacewing::RelayServer &Server, Lacewing::Error &Error);
	void OnError(Lacewing::FlashPolicy &FlashPolicy, Lacewing::Error &Error);
	void OnServerMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, bool Blasted, int Subchannel, char * Data, int Size, int Variant);
	bool OnChannelMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel, bool Blasted, int Subchannel, char * Data, int Size, int Variant);
	bool OnPeerMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel, Lacewing::RelayServer::Client &TargetClient, bool Blasted, int Subchannel, char * Packet, int Size, int Variant);
	bool OnJoinChannel(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel);
	bool OnLeaveChannel(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel);
	bool OnSetName(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, const char * Name);
};
// File server: Manages file transfers
class SERVER_FILE : public SERVER
{
public:
	// Variables (stripped)
	HANDLE SendingThread;	// Handle to file sending thread
	
	// Pass along
	bool OnConnect(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client);
	void OnDisconnect(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client);
	void OnError(Lacewing::RelayServer &Server, Lacewing::Error &Error);
	void OnError(Lacewing::FlashPolicy &FlashPolicy, Lacewing::Error &Error);
	void OnServerMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, bool Blasted, int Subchannel, char * Data, int Size, int Variant);
	bool OnChannelMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel, bool Blasted, int Subchannel, char * Data, int Size, int Variant);
	bool OnPeerMessage(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel, Lacewing::RelayServer::Client &TargetClient, bool Blasted, int Subchannel, char * Packet, int Size, int Variant);
	bool OnJoinChannel(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel);
	bool OnLeaveChannel(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, Lacewing::RelayServer::Channel &Channel);
	bool OnSetName(Lacewing::RelayServer &Server, Lacewing::RelayServer::Client &Client, const char * Name);
	
	// Functions
	SERVER_FILE(Extension * E, char * FlashPolicy);
	~SERVER_FILE();
};

// End guard
#endif //EXTRACTED_SERVER_H